@@ -4,16 +4,16 @@
 #[macro_use]
 extern crate lazy_static;
 
-pub use crate::value::to_value;
+use crate::value::{to_value, Value};
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 use std::cmp::Ordering;
 use std::collections::{BTreeMap, HashSet};
 use std::convert::{TryFrom, TryInto};
 use std::fmt;
-use std::hash::{Hash, Hasher};
+use std::hash::{BuildHasher, Hash, Hasher};
 use std::str::FromStr;
-pub use value::Value;
+use std::time::Duration;
 
 pub const LOG_LEVEL_TRACE: u8 = 0;
 pub const LOG_LEVEL_DEBUG: u8 = 10;
@@ -21,27 +21,42 @@ pub const LOG_LEVEL_INFO: u8 = 20;
 pub const LOG_LEVEL_WARN: u8 = 30;
 pub const LOG_LEVEL_ERROR: u8 = 40;
 
-#[path = "tools.rs"]
+pub const DEFAULT_TIMEOUT: Duration = Duration::from_secs(5);
+
 pub mod tools;
 
 #[cfg(feature = "acl")]
-#[path = "acl.rs"]
 pub mod acl;
+#[cfg(feature = "actions")]
+pub mod actions;
+#[cfg(feature = "cache")]
+pub mod cache;
+#[cfg(feature = "events")]
+pub mod events;
 #[cfg(feature = "ext")]
-#[path = "ext.rs"]
 pub mod ext;
+#[cfg(feature = "hyper")]
+pub mod hyper_tools;
+#[cfg(feature = "registry")]
+pub mod registry;
+#[cfg(feature = "services")]
+pub mod services;
+#[cfg(feature = "time")]
+pub mod time;
+#[cfg(feature = "workers")]
+pub mod workers;
 
-#[path = "value/value.rs"]
 pub mod value;
 
 pub mod prelude {
-    //pub use crate::tools::VecX;
     pub use crate::value::to_value;
     pub use crate::value::Value;
-    pub use crate::value::ValueGet;
-    pub use crate::EvaError;
-    pub use crate::EvaErrorKind;
-    pub use crate::EvaItemKind;
+    pub use crate::value::ValueOption;
+    pub use crate::value::ValueOptionOwned;
+    pub use crate::EResult;
+    pub use crate::Error;
+    pub use crate::ErrorKind;
+    pub use crate::ItemKind;
     pub use crate::ItemStatus;
     pub use crate::IEID;
     pub use crate::OID;
@@ -50,7 +65,14 @@ pub mod prelude {
 static ERR_INVALID_OID: &str = "Invalid OID format";
 static ERR_OID_TOO_LONG: &str = "OID too long";
 
-pub type EvaResult<T> = std::result::Result<T, EvaError>;
+pub const SLEEP_STEP: Duration = Duration::from_millis(100);
+
+#[inline]
+pub fn get_default_sleep_step() -> Duration {
+    SLEEP_STEP
+}
+
+pub type EResult<T> = std::result::Result<T, Error>;
 
 pub type ItemStatus = i16;
 
@@ -90,9 +112,18 @@ pub const ERR_CODE_ELBUS_BUSY: i16 = -32118;
 pub const ERR_CODE_ELBUS_NOT_DELIVERED: i16 = -32119;
 pub const ERR_CODE_ELBUS_TIMEOUT: i16 = -32120;
 
+#[inline]
+pub fn is_str_wildcard(s: &str) -> bool {
+    s == "#" || s == "*"
+}
+#[inline]
+pub fn is_str_any(s: &str) -> bool {
+    s == "+" || s == "?"
+}
+
 #[derive(Serialize_repr, Deserialize_repr, Eq, PartialEq, Debug, Copy, Clone)]
 #[repr(i16)]
-pub enum EvaErrorKind {
+pub enum ErrorKind {
     CoreError = ERR_CODE_CORE_ERROR,
     Unsupported = ERR_CODE_UNSUPPORTED,
     NotReady = ERR_CODE_NOT_READY,
@@ -122,142 +153,197 @@ pub enum EvaErrorKind {
     Other = ERR_CODE_OTHER,
 }
 
-impl From<i16> for EvaErrorKind {
-    fn from(code: i16) -> EvaErrorKind {
+impl From<i16> for ErrorKind {
+    fn from(code: i16) -> ErrorKind {
         match code {
-            x if x == EvaErrorKind::CoreError as i16 => EvaErrorKind::CoreError,
-            x if x == EvaErrorKind::Unsupported as i16 => EvaErrorKind::Unsupported,
-            x if x == EvaErrorKind::IOError as i16 => EvaErrorKind::IOError,
-            x if x == EvaErrorKind::RegistryError as i16 => EvaErrorKind::RegistryError,
-            x if x == EvaErrorKind::InvalidData as i16 => EvaErrorKind::InvalidData,
-            x if x == EvaErrorKind::FunctionFailed as i16 => EvaErrorKind::FunctionFailed,
-            x if x == EvaErrorKind::ResourceNotFound as i16 => EvaErrorKind::ResourceNotFound,
-            x if x == EvaErrorKind::ResourceBusy as i16 => EvaErrorKind::ResourceBusy,
-            x if x == EvaErrorKind::ResourceAlreadyExists as i16 => {
-                EvaErrorKind::ResourceAlreadyExists
+            x if x == ErrorKind::CoreError as i16 => ErrorKind::CoreError,
+            x if x == ErrorKind::Unsupported as i16 => ErrorKind::Unsupported,
+            x if x == ErrorKind::IOError as i16 => ErrorKind::IOError,
+            x if x == ErrorKind::RegistryError as i16 => ErrorKind::RegistryError,
+            x if x == ErrorKind::InvalidData as i16 => ErrorKind::InvalidData,
+            x if x == ErrorKind::FunctionFailed as i16 => ErrorKind::FunctionFailed,
+            x if x == ErrorKind::ResourceNotFound as i16 => ErrorKind::ResourceNotFound,
+            x if x == ErrorKind::ResourceBusy as i16 => ErrorKind::ResourceBusy,
+            x if x == ErrorKind::ResourceAlreadyExists as i16 => ErrorKind::ResourceAlreadyExists,
+            x if x == ErrorKind::AccessDenied as i16 => ErrorKind::AccessDenied,
+            x if x == ErrorKind::MethodNotImplemented as i16 => ErrorKind::MethodNotImplemented,
+            x if x == ErrorKind::MethodNotFound as i16 => ErrorKind::MethodNotFound,
+            x if x == ErrorKind::InvalidParameter as i16 => ErrorKind::InvalidParameter,
+            x if x == ErrorKind::Timeout as i16 => ErrorKind::Timeout,
+            x if x == ErrorKind::Aborted as i16 => ErrorKind::Aborted,
+            x if x == ErrorKind::EvaHIAuthenticationRequired as i16 => {
+                ErrorKind::EvaHIAuthenticationRequired
             }
-            x if x == EvaErrorKind::AccessDenied as i16 => EvaErrorKind::AccessDenied,
-            x if x == EvaErrorKind::MethodNotImplemented as i16 => {
-                EvaErrorKind::MethodNotImplemented
+            x if x == ErrorKind::TokenRestricted as i16 => ErrorKind::TokenRestricted,
+            x if x == ErrorKind::Other as i16 => ErrorKind::Other,
+            x if x == ErrorKind::NotReady as i16 => ErrorKind::NotReady,
+            x if x == ErrorKind::ElbusClientNotRegistered as i16 => {
+                ErrorKind::ElbusClientNotRegistered
             }
-            x if x == EvaErrorKind::MethodNotFound as i16 => EvaErrorKind::MethodNotFound,
-            x if x == EvaErrorKind::InvalidParameter as i16 => EvaErrorKind::InvalidParameter,
-            x if x == EvaErrorKind::Timeout as i16 => EvaErrorKind::Timeout,
-            x if x == EvaErrorKind::Aborted as i16 => EvaErrorKind::Aborted,
-            x if x == EvaErrorKind::EvaHIAuthenticationRequired as i16 => {
-                EvaErrorKind::EvaHIAuthenticationRequired
-            }
-            x if x == EvaErrorKind::TokenRestricted as i16 => EvaErrorKind::TokenRestricted,
-            x if x == EvaErrorKind::Other as i16 => EvaErrorKind::Other,
-            x if x == EvaErrorKind::NotReady as i16 => EvaErrorKind::NotReady,
-            x if x == EvaErrorKind::ElbusClientNotRegistered as i16 => {
-                EvaErrorKind::ElbusClientNotRegistered
-            }
-            x if x == EvaErrorKind::ElbusData as i16 => EvaErrorKind::ElbusData,
-            x if x == EvaErrorKind::ElbusIo as i16 => EvaErrorKind::ElbusIo,
-            x if x == EvaErrorKind::ElbusOther as i16 => EvaErrorKind::ElbusOther,
-            x if x == EvaErrorKind::ElbusNotSupported as i16 => EvaErrorKind::ElbusNotSupported,
-            x if x == EvaErrorKind::ElbusBusy as i16 => EvaErrorKind::ElbusBusy,
-            x if x == EvaErrorKind::ElbusNotDelivered as i16 => EvaErrorKind::ElbusNotDelivered,
-            x if x == EvaErrorKind::ElbusTimeout as i16 => EvaErrorKind::ElbusTimeout,
-            _ => EvaErrorKind::Other,
+            x if x == ErrorKind::ElbusData as i16 => ErrorKind::ElbusData,
+            x if x == ErrorKind::ElbusIo as i16 => ErrorKind::ElbusIo,
+            x if x == ErrorKind::ElbusOther as i16 => ErrorKind::ElbusOther,
+            x if x == ErrorKind::ElbusNotSupported as i16 => ErrorKind::ElbusNotSupported,
+            x if x == ErrorKind::ElbusBusy as i16 => ErrorKind::ElbusBusy,
+            x if x == ErrorKind::ElbusNotDelivered as i16 => ErrorKind::ElbusNotDelivered,
+            x if x == ErrorKind::ElbusTimeout as i16 => ErrorKind::ElbusTimeout,
+            _ => ErrorKind::Other,
         }
     }
 }
 
-impl std::fmt::Display for EvaErrorKind {
+impl std::fmt::Display for ErrorKind {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
             "{}",
             match self {
-                EvaErrorKind::CoreError => "Core error",
-                EvaErrorKind::Unsupported => "Unsupported",
-                EvaErrorKind::IOError => "IO error",
-                EvaErrorKind::RegistryError => "Registry error",
-                EvaErrorKind::InvalidData => "Invalid data",
-                EvaErrorKind::FunctionFailed => "Function failed",
-                EvaErrorKind::ResourceNotFound => "Resource not found",
-                EvaErrorKind::ResourceBusy => "Resource busy",
-                EvaErrorKind::ResourceAlreadyExists => "Resource already exists",
-                EvaErrorKind::AccessDenied => "Access denied",
-                EvaErrorKind::MethodNotImplemented => "Method not implemented",
-                EvaErrorKind::MethodNotFound => "Method not found",
-                EvaErrorKind::InvalidParameter => "Invalid parameter",
-                EvaErrorKind::Timeout => "Timed out",
-                EvaErrorKind::Aborted => "Aborted",
-                EvaErrorKind::EvaHIAuthenticationRequired => "EvaHI authentication required",
-                EvaErrorKind::TokenRestricted => "Token restricted",
-                EvaErrorKind::Other => "Other",
-                EvaErrorKind::NotReady => "Not ready",
-                EvaErrorKind::ElbusClientNotRegistered => "ELBUS client not registered",
-                EvaErrorKind::ElbusData => "ELBUS data error",
-                EvaErrorKind::ElbusIo => "ELBUS IO error",
-                EvaErrorKind::ElbusOther => "ELBUS error",
-                EvaErrorKind::ElbusNotSupported => "ELBUS feature not supported",
-                EvaErrorKind::ElbusBusy => "ELBUS busy",
-                EvaErrorKind::ElbusNotDelivered => "ELBUS not delivered",
-                EvaErrorKind::ElbusTimeout => "ELBUS timed out",
+                ErrorKind::CoreError => "Core error",
+                ErrorKind::Unsupported => "Unsupported",
+                ErrorKind::IOError => "IO error",
+                ErrorKind::RegistryError => "Registry error",
+                ErrorKind::InvalidData => "Invalid data",
+                ErrorKind::FunctionFailed => "Function failed",
+                ErrorKind::ResourceNotFound => "Resource not found",
+                ErrorKind::ResourceBusy => "Resource busy",
+                ErrorKind::ResourceAlreadyExists => "Resource already exists",
+                ErrorKind::AccessDenied => "Access denied",
+                ErrorKind::MethodNotImplemented => "Method not implemented",
+                ErrorKind::MethodNotFound => "Method not found",
+                ErrorKind::InvalidParameter => "Invalid parameter",
+                ErrorKind::Timeout => "Timed out",
+                ErrorKind::Aborted => "Aborted",
+                ErrorKind::EvaHIAuthenticationRequired => "EvaHI authentication required",
+                ErrorKind::TokenRestricted => "Token restricted",
+                ErrorKind::Other => "Other",
+                ErrorKind::NotReady => "Not ready",
+                ErrorKind::ElbusClientNotRegistered => "ELBUS client not registered",
+                ErrorKind::ElbusData => "ELBUS data error",
+                ErrorKind::ElbusIo => "ELBUS IO error",
+                ErrorKind::ElbusOther => "ELBUS error",
+                ErrorKind::ElbusNotSupported => "ELBUS feature not supported",
+                ErrorKind::ElbusBusy => "ELBUS busy",
+                ErrorKind::ElbusNotDelivered => "ELBUS not delivered",
+                ErrorKind::ElbusTimeout => "ELBUS timed out",
             }
         )
     }
 }
 
 #[derive(Debug, Eq, PartialEq)]
-pub struct EvaError {
-    kind: EvaErrorKind,
+pub struct Error {
+    kind: ErrorKind,
     message: Option<String>,
 }
 
-impl std::error::Error for EvaError {}
+impl std::error::Error for Error {}
 
-#[cfg(feature = "ext")]
-impl From<rmp_serde::encode::Error> for EvaError {
-    fn from(error: rmp_serde::encode::Error) -> EvaError {
-        EvaError::invalid_data(error)
+macro_rules! impl_err_error {
+    ($src: ty, $f: path) => {
+        impl From<$src> for Error {
+            fn from(err: $src) -> Error {
+                $f(err)
+            }
+        }
+    };
+}
+
+impl_err_error!(std::string::FromUtf8Error, Error::invalid_data);
+impl_err_error!(std::num::ParseIntError, Error::invalid_data);
+impl_err_error!(std::num::ParseFloatError, Error::invalid_data);
+impl_err_error!(ipnetwork::IpNetworkError, Error::invalid_data);
+impl_err_error!(serde_json::Error, Error::invalid_data);
+impl_err_error!(std::io::Error, Error::io);
+#[cfg(feature = "elbus-rpc")]
+impl_err_error!(elbus::Error, Error::io);
+#[cfg(any(feature = "services", feature = "workers"))]
+impl_err_error!(tokio::sync::oneshot::error::RecvError, Error::io);
+#[cfg(any(feature = "services", feature = "workers"))]
+impl_err_error!(tokio::sync::TryLockError, Error::core);
+#[cfg(any(
+    feature = "elbus-rpc",
+    feature = "registry",
+    feature = "ext",
+    feature = "cache"
+))]
+impl_err_error!(rmp_serde::encode::Error, Error::invalid_data);
+#[cfg(any(
+    feature = "elbus-rpc",
+    feature = "registry",
+    feature = "ext",
+    feature = "cache"
+))]
+impl_err_error!(rmp_serde::decode::Error, Error::invalid_data);
+impl_err_error!(std::array::TryFromSliceError, Error::invalid_data);
+#[cfg(feature = "db")]
+impl_err_error!(yedb::Error, Error::registry);
+#[cfg(any(feature = "db", feature = "cache"))]
+impl_err_error!(sqlx::Error, Error::io);
+#[cfg(feature = "dataconv")]
+impl_err_error!(hex::FromHexError, Error::invalid_data);
+#[cfg(feature = "dataconv")]
+impl_err_error!(regex::Error, Error::invalid_data);
+#[cfg(any(feature = "actions", feature = "dataconv"))]
+impl_err_error!(uuid::Error, Error::invalid_data);
+
+#[cfg(feature = "elbus-rpc")]
+impl From<elbus::rpc::RpcError> for Error {
+    fn from(err: elbus::rpc::RpcError) -> Self {
+        Error {
+            kind: err.code().into(),
+            message: err
+                .data()
+                .map(|v| std::str::from_utf8(v).unwrap_or_default().to_owned()),
+        }
     }
 }
 
-#[cfg(feature = "ext")]
-impl From<rmp_serde::decode::Error> for EvaError {
-    fn from(error: rmp_serde::decode::Error) -> EvaError {
-        EvaError::invalid_data(error)
+#[cfg(feature = "elbus-rpc")]
+impl From<Error> for elbus::rpc::RpcError {
+    fn from(err: Error) -> Self {
+        elbus::rpc::RpcError::new(
+            err.kind() as i16,
+            elbus::rpc::rpc_err_str(err.message().unwrap_or_default()),
+        )
     }
 }
 
-impl From<std::num::ParseIntError> for EvaError {
-    fn from(err: std::num::ParseIntError) -> EvaError {
-        EvaError::invalid_data(err)
+#[cfg(feature = "elbus-rpc")]
+impl From<crate::value::SerializerError> for elbus::rpc::RpcError {
+    fn from(err: crate::value::SerializerError) -> Self {
+        elbus::rpc::RpcError::new(
+            ErrorKind::InvalidData as i16,
+            elbus::rpc::rpc_err_str(err.to_string()),
+        )
     }
 }
 
-impl From<std::num::ParseFloatError> for EvaError {
-    fn from(err: std::num::ParseFloatError) -> EvaError {
-        EvaError::invalid_data(err)
+#[cfg(feature = "elbus-rpc")]
+impl From<crate::value::DeserializerError> for elbus::rpc::RpcError {
+    fn from(err: crate::value::DeserializerError) -> Self {
+        elbus::rpc::RpcError::new(
+            ErrorKind::InvalidData as i16,
+            elbus::rpc::rpc_err_str(err.to_string()),
+        )
     }
 }
 
-impl From<std::convert::Infallible> for EvaError {
-    fn from(_err: std::convert::Infallible) -> EvaError {
+#[cfg(any(feature = "services", feature = "workers"))]
+impl From<tokio::time::error::Elapsed> for Error {
+    fn from(_e: tokio::time::error::Elapsed) -> Error {
+        Error::timeout()
+    }
+}
+
+impl From<std::convert::Infallible> for Error {
+    fn from(_err: std::convert::Infallible) -> Error {
         panic!();
     }
 }
 
-impl From<ipnetwork::IpNetworkError> for EvaError {
-    fn from(err: ipnetwork::IpNetworkError) -> EvaError {
-        EvaError::invalid_data(err)
-    }
-}
-
-impl From<serde_json::Error> for EvaError {
-    fn from(error: serde_json::Error) -> Self {
-        EvaError::invalid_data(error).into()
-    }
-}
-
-impl EvaError {
+impl Error {
     #[allow(clippy::must_use_candidate)]
-    pub fn new<T: fmt::Display>(kind: EvaErrorKind, message: T) -> Self {
+    pub fn new<T: fmt::Display>(kind: ErrorKind, message: T) -> Self {
         Self {
             kind,
             message: Some(message.to_string()),
@@ -265,7 +351,7 @@ impl EvaError {
     }
 
     #[allow(clippy::must_use_candidate)]
-    pub fn new0(kind: EvaErrorKind) -> Self {
+    pub fn new0(kind: ErrorKind) -> Self {
         Self {
             kind,
             message: None,
@@ -273,7 +359,7 @@ impl EvaError {
     }
 
     #[allow(clippy::must_use_candidate)]
-    pub fn newc(kind: EvaErrorKind, message: Option<String>) -> Self {
+    pub fn newc(kind: ErrorKind, message: Option<String>) -> Self {
         Self { kind, message }
     }
 
@@ -282,7 +368,7 @@ impl EvaError {
     }
 
     #[allow(clippy::must_use_candidate)]
-    pub fn e<T: fmt::Display>(kind: EvaErrorKind, message: T) -> Self {
+    pub fn e<T: fmt::Display>(kind: ErrorKind, message: T) -> Self {
         Self {
             kind,
             message: Some(message.to_string()),
@@ -292,7 +378,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn not_found<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::ResourceNotFound,
+            kind: ErrorKind::ResourceNotFound,
             message: Some(message.to_string()),
         }
     }
@@ -300,7 +386,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn not_ready<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::NotReady,
+            kind: ErrorKind::NotReady,
             message: Some(message.to_string()),
         }
     }
@@ -308,7 +394,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn unsupported<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::Unsupported,
+            kind: ErrorKind::Unsupported,
             message: Some(message.to_string()),
         }
     }
@@ -316,7 +402,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn registry<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::RegistryError,
+            kind: ErrorKind::RegistryError,
             message: Some(message.to_string()),
         }
     }
@@ -324,7 +410,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn busy<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::ResourceBusy,
+            kind: ErrorKind::ResourceBusy,
             message: Some(message.to_string()),
         }
     }
@@ -332,7 +418,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn core<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::CoreError,
+            kind: ErrorKind::CoreError,
             message: Some(message.to_string()),
         }
     }
@@ -340,7 +426,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn io<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::IOError,
+            kind: ErrorKind::IOError,
             message: Some(message.to_string()),
         }
     }
@@ -348,7 +434,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn duplicate<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::ResourceAlreadyExists,
+            kind: ErrorKind::ResourceAlreadyExists,
             message: Some(message.to_string()),
         }
     }
@@ -356,7 +442,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn failed<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::FunctionFailed,
+            kind: ErrorKind::FunctionFailed,
             message: Some(message.to_string()),
         }
     }
@@ -364,7 +450,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn access<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::AccessDenied,
+            kind: ErrorKind::AccessDenied,
             message: Some(message.to_string()),
         }
     }
@@ -372,7 +458,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn timeout() -> Self {
         Self {
-            kind: EvaErrorKind::Timeout,
+            kind: ErrorKind::Timeout,
             message: None,
         }
     }
@@ -380,7 +466,7 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn aborted() -> Self {
         Self {
-            kind: EvaErrorKind::Timeout,
+            kind: ErrorKind::Timeout,
             message: None,
         }
     }
@@ -388,12 +474,23 @@ impl EvaError {
     #[allow(clippy::must_use_candidate)]
     pub fn invalid_data<T: fmt::Display>(message: T) -> Self {
         Self {
-            kind: EvaErrorKind::InvalidData,
+            kind: ErrorKind::InvalidData,
             message: Some(message.to_string()),
         }
     }
-
-    pub fn kind(&self) -> EvaErrorKind {
+    pub fn invalid_params<T: fmt::Display>(message: T) -> Self {
+        Self {
+            kind: ErrorKind::InvalidParameter,
+            message: Some(message.to_string()),
+        }
+    }
+    pub fn not_implemented<T: fmt::Display>(message: T) -> Self {
+        Self {
+            kind: ErrorKind::MethodNotImplemented,
+            message: Some(message.to_string()),
+        }
+    }
+    pub fn kind(&self) -> ErrorKind {
         self.kind
     }
     pub fn message(&self) -> Option<&str> {
@@ -401,7 +498,7 @@ impl EvaError {
     }
 }
 
-impl std::fmt::Display for EvaError {
+impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut err = self.kind.to_string();
         if let Some(msg) = self.message.as_ref() {
@@ -429,6 +526,9 @@ impl IEID {
         self.0 = 0;
         self.1 = 0;
     }
+    /// # Panics
+    ///
+    /// Will panic if the serializer has gone mad
     #[allow(clippy::must_use_candidate)]
     #[inline]
     pub fn to_value(&self) -> Value {
@@ -451,28 +551,28 @@ impl IEID {
 }
 
 impl TryFrom<&Value> for IEID {
-    type Error = EvaError;
-    fn try_from(v: &Value) -> EvaResult<Self> {
+    type Error = Error;
+    fn try_from(v: &Value) -> EResult<Self> {
         if let Value::Seq(s) = v {
             let mut ix = s.iter();
             let ieid_b = if let Some(b) = ix.next() {
                 b.try_into()?
             } else {
-                return Err(EvaError::invalid_data("First IEID element mismatch"));
+                return Err(Error::invalid_data("First IEID element mismatch"));
             };
             let ieid_i = if let Some(i) = ix.next() {
                 i.try_into()?
             } else {
-                return Err(EvaError::invalid_data("Second IEID element mismatch"));
+                return Err(Error::invalid_data("Second IEID element mismatch"));
             };
             if ix.next().is_some() {
-                return Err(EvaError::invalid_data(
+                return Err(Error::invalid_data(
                     "Incompatible IEID (more than 2 elements)",
                 ));
             }
             Ok(Self(ieid_b, ieid_i))
         } else {
-            Err(EvaError::invalid_data("invalid value for IEID"))
+            Err(Error::invalid_data("invalid value for IEID"))
         }
     }
 }
@@ -489,7 +589,7 @@ impl PartialOrd for IEID {
 
 #[derive(Clone, Eq)]
 pub struct OID {
-    kind: EvaItemKind,
+    kind: ItemKind,
     oid_str: String,
     tpos: u16,
     grp_pos: Option<u16>,
@@ -504,7 +604,7 @@ impl PartialEq for OID {
 impl Ord for OID {
     fn cmp(&self, other: &Self) -> Ordering {
         if self.kind == other.kind {
-            self.oid_str.cmp(&other.oid_str)
+            self.full_id().cmp(other.full_id())
         } else {
             self.kind.cmp(&other.kind)
         }
@@ -517,34 +617,34 @@ impl PartialOrd for OID {
     }
 }
 
-const OID_ALLOWED_SYMBOLS: &str = "_.()[]-+\\";
+pub const OID_ALLOWED_SYMBOLS: &str = "_.()[]-\\";
+pub const OID_MASK_ALLOWED_SYMBOLS: &str = "_.()[]-+?#*\\";
 
 impl OID {
     #[inline]
-    fn check_allowed_symbols(s: &str, is_path: bool) -> EvaResult<()> {
+    fn check(s: &str, is_path: bool) -> EResult<()> {
+        if s.len() > 65000 {
+            return Err(Error::invalid_data("OID too long"));
+        }
         for c in s.chars() {
-            if !c.is_alphanumeric() && !(is_path && c == '/') && !OID_ALLOWED_SYMBOLS.contains(c) {
-                return Err(EvaError::invalid_data(format!(
-                    "Invalid symbol in OID: {}",
-                    c
-                )));
+            if !(c.is_alphanumeric() || OID_ALLOWED_SYMBOLS.contains(c) || (is_path && c == '/')) {
+                return Err(Error::invalid_data(format!("Invalid symbol in OID: {}", c)));
             }
         }
         Ok(())
     }
-    pub fn new(kind: EvaItemKind, group: &str, id: &str) -> EvaResult<Self> {
-        OID::check_allowed_symbols(group, true)?;
-        OID::check_allowed_symbols(id, false)?;
+    #[allow(clippy::cast_possible_truncation)]
+    pub fn new(kind: ItemKind, group: &str, id: &str) -> EResult<Self> {
+        OID::check(group, true)?;
+        OID::check(id, false)?;
         if group == "+" || id == "+" {
-            return Err(EvaError::invalid_data(
-                "OID group or id can not be equal to +",
-            ));
+            return Err(Error::invalid_data("OID group or id can not be equal to +"));
         }
         let tp_str = kind.to_string();
         if id.is_empty() || group.is_empty() {
-            Err(EvaError::invalid_data(ERR_INVALID_OID))
+            Err(Error::invalid_data(ERR_INVALID_OID))
         } else if tp_str.len() + id.len() + group.len() + 2 > std::u16::MAX as usize {
-            Err(EvaError::invalid_data(ERR_OID_TOO_LONG))
+            Err(Error::invalid_data(ERR_OID_TOO_LONG))
         } else {
             let oid_str = format!("{}:{}/{}", kind, group, id);
             let grp_pos = Some((group.len() as u16) + (tp_str.len() as u16) + 1);
@@ -556,13 +656,24 @@ impl OID {
             })
         }
     }
-    pub fn new0(kind: EvaItemKind, id: &str) -> EvaResult<Self> {
-        OID::check_allowed_symbols(id, true)?;
+    #[inline]
+    pub fn new0(kind: ItemKind, id: &str) -> EResult<Self> {
+        Self::_new0(kind, id, true)
+    }
+    #[inline]
+    pub fn new0_unchecked(kind: ItemKind, id: &str) -> EResult<Self> {
+        Self::_new0(kind, id, false)
+    }
+    #[allow(clippy::cast_possible_truncation)]
+    fn _new0(kind: ItemKind, id: &str, need_check: bool) -> EResult<Self> {
+        if need_check {
+            OID::check(id, true)?;
+        }
         let tp_str = kind.to_string();
         if id.is_empty() {
-            Err(EvaError::invalid_data(ERR_INVALID_OID))
+            Err(Error::invalid_data(ERR_INVALID_OID))
         } else if id.len() + tp_str.len() >= std::u16::MAX as usize {
-            Err(EvaError::invalid_data(ERR_OID_TOO_LONG))
+            Err(Error::invalid_data(ERR_OID_TOO_LONG))
         } else {
             let grp_pos = id.rfind('/').map(|p| p as u16 + tp_str.len() as u16 + 1);
             let oid_str = format!("{}:{}", kind, id);
@@ -591,7 +702,7 @@ impl OID {
             .map(|g| &self.oid_str[self.tpos as usize..g as usize])
     }
     #[inline]
-    pub fn kind(&self) -> EvaItemKind {
+    pub fn kind(&self) -> ItemKind {
         self.kind
     }
     #[inline]
@@ -602,25 +713,33 @@ impl OID {
     pub fn as_str(&self) -> &str {
         self.oid_str.as_str()
     }
+    #[inline]
+    pub fn is_wildcard(&self) -> bool {
+        is_str_wildcard(self.id())
+    }
+    #[inline]
+    pub fn to_wildcard_str(&self, wildcard_suffix: &str) -> String {
+        let mut s = format!("{}:", self.kind);
+        if let Some(group) = self.group() {
+            s = s + group + "/";
+        }
+        s + wildcard_suffix
+    }
     pub fn serialize_into(&self, target: &mut BTreeMap<Value, Value>) {
         target.insert("oid".into(), self.as_str().into());
         //COMPAT, deprecated, remove in 4.2
         target.insert("full_id".into(), self.full_id().into());
         target.insert("id".into(), self.id().into());
-        target.insert(
-            "group".into(),
-            self.group()
-                .map_or_else(|| Value::Option(None), |g| g.into()),
-        );
+        target.insert("group".into(), self.group().map_or(Value::Unit, Into::into));
         target.insert("type".into(), self.kind.into());
     }
-    pub fn from_str_type(tp: EvaItemKind, s: &str) -> EvaResult<Self> {
+    pub fn from_str_type(tp: ItemKind, s: &str) -> EResult<Self> {
         if let Some(tpos) = s.find(':') {
-            let otp: EvaItemKind = s[..tpos].parse()?;
+            let otp: ItemKind = s[..tpos].parse()?;
             if otp == tp {
                 Self::new0(tp, &s[tpos + 1..])
             } else {
-                Err(EvaError::invalid_data(format!(
+                Err(Error::invalid_data(format!(
                     "OID type mismatch, expected: {}, found: {}",
                     tp, otp
                 )))
@@ -629,26 +748,32 @@ impl OID {
             OID::new0(tp, s)
         }
     }
-}
-
-impl FromStr for OID {
-    type Err = EvaError;
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        s.find([':', '/']).map_or(
-            Err(EvaError::invalid_data(format!(
-                "{}: {}",
-                ERR_INVALID_OID, s
-            ))),
+    #[inline]
+    pub fn from_path(s: &str) -> EResult<Self> {
+        Self::parse_oid(s, '/')
+    }
+    #[inline]
+    fn parse_oid(s: &str, c: char) -> EResult<Self> {
+        s.find(c).map_or(
+            Err(Error::invalid_data(format!("{}: {}", ERR_INVALID_OID, s))),
             |tpos| {
-                let tp: EvaItemKind = s[..tpos].parse()?;
+                let tp: ItemKind = s[..tpos].parse()?;
                 Self::new0(tp, &s[tpos + 1..])
             },
         )
     }
 }
 
+impl FromStr for OID {
+    type Err = Error;
+    #[inline]
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Self::parse_oid(s, ':')
+    }
+}
+
 impl TryFrom<&Value> for OID {
-    type Error = EvaError;
+    type Error = Error;
     fn try_from(value: &Value) -> Result<OID, Self::Error> {
         let s: &str = value.try_into()?;
         s.parse()
@@ -718,102 +843,102 @@ impl From<&OID> for Value {
 }
 
 impl TryFrom<Value> for OID {
-    type Error = EvaError;
-    fn try_from(value: Value) -> EvaResult<OID> {
+    type Error = Error;
+    fn try_from(value: Value) -> EResult<OID> {
         match value {
             Value::String(s) => Ok(s.parse()?),
-            _ => Err(EvaError::invalid_data("Expected string")),
+            _ => Err(Error::invalid_data("Expected string")),
         }
     }
 }
 
-impl TryFrom<Value> for HashSet<OID> {
-    type Error = EvaError;
-    fn try_from(value: Value) -> EvaResult<HashSet<OID>> {
+impl<S: BuildHasher + Default> TryFrom<Value> for HashSet<OID, S> {
+    type Error = Error;
+    fn try_from(value: Value) -> EResult<HashSet<OID, S>> {
         match value {
             Value::Seq(vec) => {
-                let mut result = HashSet::new();
+                let mut result = HashSet::default();
                 for v in vec {
                     result.insert(v.try_into()?);
                 }
                 Ok(result)
             }
             Value::String(s) => {
-                let mut result = HashSet::new();
+                let mut result = HashSet::default();
                 for v in s.split(',') {
                     result.insert(v.parse()?);
                 }
                 Ok(result)
             }
-            _ => Err(EvaError::invalid_data("Expected vec or string")),
+            _ => Err(Error::invalid_data("Expected vec or string")),
         }
     }
 }
 
-impl From<HashSet<OID>> for Value {
-    fn from(v: HashSet<OID>) -> Value {
+impl<S: BuildHasher> From<HashSet<OID, S>> for Value {
+    fn from(v: HashSet<OID, S>) -> Value {
         Value::Seq(v.iter().map(|oid| to_value(oid).unwrap()).collect())
     }
 }
 
 #[derive(Debug, Eq, PartialEq, Copy, Clone, Ord, PartialOrd, Hash)]
 #[repr(u16)]
-pub enum EvaItemKind {
+pub enum ItemKind {
     Unit = 100,
     Sensor = 101,
     Lvar = 200,
     Lmacro = 300,
 }
 
-impl EvaItemKind {
+impl ItemKind {
     pub fn as_str(&self) -> &str {
         match self {
-            EvaItemKind::Unit => "unit",
-            EvaItemKind::Sensor => "sensor",
-            EvaItemKind::Lvar => "lvar",
-            EvaItemKind::Lmacro => "lmacro",
+            ItemKind::Unit => "unit",
+            ItemKind::Sensor => "sensor",
+            ItemKind::Lvar => "lvar",
+            ItemKind::Lmacro => "lmacro",
         }
     }
 }
 
-impl fmt::Display for EvaItemKind {
+impl fmt::Display for ItemKind {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.as_str())
     }
 }
 
-impl From<EvaItemKind> for Value {
-    fn from(src: EvaItemKind) -> Value {
+impl From<ItemKind> for Value {
+    fn from(src: ItemKind) -> Value {
         src.to_string().into()
     }
 }
 
-impl FromStr for EvaItemKind {
-    type Err = EvaError;
+impl FromStr for ItemKind {
+    type Err = Error;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         match s {
-            "unit" | "U" => Ok(EvaItemKind::Unit),
-            "sensor" | "S" => Ok(EvaItemKind::Sensor),
-            "lvar" => Ok(EvaItemKind::Lvar),
-            "lmacro" | "K" => Ok(EvaItemKind::Lmacro),
-            _ => Err(EvaError::new(
-                EvaErrorKind::InvalidData,
+            "unit" | "U" => Ok(ItemKind::Unit),
+            "sensor" | "S" => Ok(ItemKind::Sensor),
+            "lvar" | "LV" => Ok(ItemKind::Lvar),
+            "lmacro" | "K" => Ok(ItemKind::Lmacro),
+            _ => Err(Error::new(
+                ErrorKind::InvalidData,
                 format!("Invalid item type: {}", s),
             )),
         }
     }
 }
 
-impl TryFrom<&Value> for EvaItemKind {
-    type Error = EvaError;
-    fn try_from(value: &Value) -> Result<EvaItemKind, Self::Error> {
+impl TryFrom<&Value> for ItemKind {
+    type Error = Error;
+    fn try_from(value: &Value) -> Result<ItemKind, Self::Error> {
         TryInto::<&str>::try_into(value)?.parse()
     }
 }
 
-impl TryFrom<&Value> for Vec<EvaItemKind> {
-    type Error = EvaError;
-    fn try_from(value: &Value) -> Result<Vec<EvaItemKind>, Self::Error> {
+impl TryFrom<&Value> for Vec<ItemKind> {
+    type Error = Error;
+    fn try_from(value: &Value) -> Result<Vec<ItemKind>, Self::Error> {
         let data: Vec<&str> = value.try_into()?;
         let mut result = Vec::new();
         for d in data {
@@ -823,16 +948,16 @@ impl TryFrom<&Value> for Vec<EvaItemKind> {
     }
 }
 
-impl TryFrom<Value> for EvaItemKind {
-    type Error = EvaError;
-    fn try_from(value: Value) -> Result<EvaItemKind, Self::Error> {
+impl TryFrom<Value> for ItemKind {
+    type Error = Error;
+    fn try_from(value: Value) -> Result<ItemKind, Self::Error> {
         TryInto::<String>::try_into(value)?.parse()
     }
 }
 
-impl TryFrom<Value> for Vec<EvaItemKind> {
-    type Error = EvaError;
-    fn try_from(value: Value) -> Result<Vec<EvaItemKind>, Self::Error> {
+impl TryFrom<Value> for Vec<ItemKind> {
+    type Error = Error;
+    fn try_from(value: Value) -> Result<Vec<ItemKind>, Self::Error> {
         let data: Vec<String> = value.try_into()?;
         let mut result = Vec::new();
         for d in data {
@@ -842,22 +967,10 @@ impl TryFrom<Value> for Vec<EvaItemKind> {
     }
 }
 
-#[cfg(feature = "elbus-rpc")]
-impl From<elbus::rpc::RpcError> for EvaError {
-    fn from(err: elbus::rpc::RpcError) -> Self {
-        EvaError {
-            kind: err.code().into(),
-            message: err
-                .data()
-                .map(|v| std::str::from_utf8(&v).unwrap_or_default().to_owned()),
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::to_value;
-    use super::{EvaError, EvaItemKind, Value, IEID, OID};
+    use super::{Error, ItemKind, Value, IEID, OID};
     use std::convert::TryInto;
 
     #[test]
@@ -866,25 +979,20 @@ mod tests {
         assert_eq!(oid.id(), "temp1");
         assert_eq!(oid.full_id(), "env/room1/temp1");
         assert_eq!(oid.group().unwrap(), "env/room1");
-        assert_eq!(oid.kind, EvaItemKind::Sensor);
-        let oid: OID = "sensor/env/room1/temp1".parse().unwrap();
-        assert_eq!(oid.id(), "temp1");
-        assert_eq!(oid.full_id(), "env/room1/temp1");
-        assert_eq!(oid.group().unwrap(), "env/room1");
-        assert_eq!(oid.kind, EvaItemKind::Sensor);
+        assert_eq!(oid.kind, ItemKind::Sensor);
         assert_eq!("sensorx:env/temp1".parse::<OID>().is_err(), true);
         assert_eq!("sensorxenv/temp1".parse::<OID>().is_err(), true);
         assert_eq!("sensorxenv/:temp1".parse::<OID>().is_err(), true);
         assert_eq!("sensor|temp1".parse::<OID>().is_err(), true);
         assert_eq!("sensor:".parse::<OID>().is_err(), true);
-        let oid = OID::new0(EvaItemKind::Sensor, "tests/test1").unwrap();
+        let oid = OID::new0(ItemKind::Sensor, "tests/test1").unwrap();
         assert_eq!(oid.id(), "test1");
         assert_eq!(oid.group().unwrap(), "tests");
-        assert_eq!(oid.kind(), EvaItemKind::Sensor);
-        let oid = OID::new0(EvaItemKind::Sensor, "tests/room1/test1").unwrap();
+        assert_eq!(oid.kind(), ItemKind::Sensor);
+        let oid = OID::new0(ItemKind::Sensor, "tests/room1/test1").unwrap();
         assert_eq!(oid.id(), "test1");
         assert_eq!(oid.group().unwrap(), "tests/room1");
-        assert_eq!(oid.kind(), EvaItemKind::Sensor);
+        assert_eq!(oid.kind(), ItemKind::Sensor);
     }
 
     #[test]
@@ -900,10 +1008,10 @@ mod tests {
     fn test_try_into_vec() {
         let v = vec!["1", "2", "3"];
         let value = to_value(v.clone()).unwrap();
-        let result: Vec<&str> = value.as_ref().try_into().unwrap();
+        let result: Vec<&str> = (&value).try_into().unwrap();
         let value2: Value = "1,2,3".into();
         assert_eq!(result, v);
-        let result: Vec<&str> = value2.as_ref().try_into().unwrap();
+        let result: Vec<&str> = (&value2).try_into().unwrap();
         assert_eq!(result, v);
     }
 
@@ -924,9 +1032,9 @@ mod tests {
 
     #[test]
     fn test_err() {
-        assert_eq!(format!("{}", EvaError::timeout()), "Timed out");
+        assert_eq!(format!("{}", Error::timeout()), "Timed out");
         assert_eq!(
-            format!("{}", EvaError::not_found("test")),
+            format!("{}", Error::not_found("test")),
             "Resource not found: test"
         );
     }
